import unittest

import conda_envfile


class Test(unittest.TestCase):
    """ """

    def test_unique(self):

        dependencies = [
            [
                ["foo =1.2", "foo >=1.2", "foo <1.3", "foo >1.0", "foo <2.0"],
                ["foo =1.2"],
            ],
            [
                ["foo =1.2", "foo >1.0", "foo <=1.3", "foo >0.9", "foo <=1.4"],
                ["foo =1.2"],
            ],
            [
                ["foo =1.2", "foo <2.0", "foo <1.3", "foo >0.0", "foo >=0.5"],
                ["foo =1.2"],
            ],
            [
                ["foo >=1.2", "foo <=1.2"],
                ["foo =1.2"],
            ],
            [
                ["foo >=1.2", "foo <=1.2", "foo ==1.2.0"],
                ["foo ==1.2.0"],
            ],
            [
                ["foo", "foo", "foo"],
                ["foo"],
            ],
            [
                ["foo", "foo >1.0", "foo >0.9", "foo >=0.8"],
                ["foo >1.0"],
            ],
            [
                ["foo", "foo <1.0", "foo <2.0"],
                ["foo <1.0"],
            ],
            [
                ["foo", "foo >1.0, <2.0", "foo >0.9, <3.0", "foo >=1.0, <=2.1", "foo >=1.0, <=2.0"],
                ["foo >1.0, <2.0"],
            ],
            [
                ["foo", "foo >=1.0, <=2.0", "foo >0.9, <3.0", "foo >=0.9, <=3.0"],
                ["foo >=1.0, <=2.0"],
            ],
            [
                ["foo *", "foo"],
                ["foo *"],
            ],
            [
                ["foo *", "foo >1.0", "foo =1.*", "foo <2.0", "foo <=3.1", "foo >0.1, <3.0"],
                ["foo >1.0, <2.0"],
            ],
            [
                ["foo =1.*", "foo >0.9", "foo", "foo <=2.0", "foo >0.1, <3.0", "foo *", "foo =1.*"],
                ["foo =1.*"],
            ],
            [
                ["foo =1.2.*", "foo", "foo <=2.0.0", "foo >1.0.1, <2.1.0", "foo =1.2.*", "foo *"],
                ["foo =1.2.*"],
            ],
            [
                ["foo =1.2.*", "foo >1.2.0", "foo >1.1.0"],
                ["foo >1.2.0, <1.3.0"],
            ],
        ]

        for deps, expect in dependencies:
            for _ in range(len(deps)):
                deps.append(deps.pop(0))
                nospace = [i.replace(" ", "") for i in deps]
                self.assertEqual(list(map(str, conda_envfile.unique(*deps))), expect)
                self.assertEqual(list(map(str, conda_envfile.unique(*nospace))), expect)

        illegal = [
            ["foo >1.2.0", "foo <1.2.0", "foo", "foo *", "foo =1.2.*"],
            ["foo =1.2.*", "foo >=1.3.0", "foo 1.*"],
            ["foo =1.2.*", "foo <1.2.0", "foo", "foo *"],
            ["foo >=1.2.0, <1.3.0", "foo >=1.3.0", "foo"],
            ["foo >=1.2.0, <2", "foo >=1.3.0", "foo >=2.0.0", "foo"],
        ]

        for deps in illegal:
            for _ in range(len(deps)):
                deps.append(deps.pop(0))
                with self.assertRaises(ValueError):
                    conda_envfile.unique(*deps)
                with self.assertRaises(ValueError):
                    conda_envfile.unique(*[i.replace(" ", "") for i in deps])

    def test_remove(self):

        self.assertEqual(conda_envfile.remove(["foo", "bar"], "bar"), ["foo"])
        self.assertEqual(conda_envfile.remove(["foo *", "bar *"], "bar"), ["foo *"])
        self.assertEqual(conda_envfile.remove(["foo =1.*", "bar =1.*"], "bar"), ["foo =1.*"])
        self.assertEqual(conda_envfile.remove(["foo >1.0", "bar >1.0"], "bar"), ["foo >1.0"])

    def test_contains(self):

        requirements = ["foo >1.0", "bar >=2.0"]
        installed = ["foo=2.0=generic", "bar=3.0=generic", "other"]
        self.assertTrue(conda_envfile.contains(requirements, installed))

        installed = ["foo=2.0=generic", "bar=1.0=generic", "other"]
        self.assertFalse(conda_envfile.contains(requirements, installed))

    def test_restrict(self):

        self.assertEqual(
            list(map(str, conda_envfile.restrict(["foo", "bar"], ["foo >1.0"]))),
            ["foo >1.0", "bar"],
        )

        self.assertEqual(
            conda_envfile.restrict(["foo", "bar"], ["foo >1.0"]),
            list(map(conda_envfile.PackageSpecifier, ["foo >1.0", "bar"])),
        )


if __name__ == "__main__":

    unittest.main(verbosity=2)



import unittest

import conda_envfile


class Test(unittest.TestCase):
    """ """

    def test_build(self):

        v = conda_envfile.PackageSpecifier("foo=1.0=pypy")
        self.assertEqual(str(v), "foo=1.0=pypy")
        self.assertEqual(v.name, "foo")
        self.assertEqual(v.version, "==1.0")
        self.assertEqual(str(v.range), "==1.0")
        self.assertEqual(v.build, "pypy")
        self.assertEqual(v.wildcard, None)

        v = conda_envfile.PackageSpecifier("foo=1.0")
        self.assertEqual(str(v), "foo =1.0")
        self.assertEqual(v.name, "foo")
        self.assertEqual(v.version, "=1.0")
        self.assertEqual(str(v.range), ">=1.0.0, <1.1.0")
        self.assertEqual(v.build, None)
        self.assertEqual(v.wildcard, "=1.0")

        v = conda_envfile.PackageSpecifier("foo=1.*")
        self.assertEqual(str(v), "foo =1.*")
        self.assertEqual(v.name, "foo")
        self.assertEqual(v.version, "=1.*")
        self.assertEqual(str(v.range), ">=1.0, <2.0")
        self.assertEqual(v.build, None)
        self.assertEqual(v.wildcard, "=1.*")

        v = conda_envfile.PackageSpecifier("foo *")
        self.assertEqual(str(v), "foo *")
        self.assertEqual(v.name, "foo")
        self.assertEqual(v.version, "*")
        self.assertEqual(str(v.range), "")
        self.assertEqual(v.build, None)
        self.assertEqual(v.wildcard, "*")

        v = conda_envfile.PackageSpecifier("foo==1.0.0")
        self.assertEqual(str(v), "foo ==1.0.0")
        self.assertEqual(v.name, "foo")
        self.assertEqual(v.version, "==1.0.0")
        self.assertEqual(str(v.range), "==1.0.0")
        self.assertEqual(v.build, None)
        self.assertEqual(v.wildcard, "==1.0.0")

    def test_merge(self):

        tests = [
            ["foo ==1.2.0", "foo =1.2", "foo ==1.2.0"],
            ["foo >=1.2, <1.3", "foo =1.2", "foo =1.2"],
        ]

        for a, b, ret in tests:
            a = conda_envfile.PackageSpecifier(a)
            b = conda_envfile.PackageSpecifier(b)
            self.assertEqual(str(a + b), ret)
            self.assertEqual(str(b + a), ret)

    def test_PackageSpecifier_in(self):

        self.assertTrue("foo ==1.0" in conda_envfile.PackageSpecifier("foo ==1.0"))
        self.assertTrue("foo =1.0" in conda_envfile.PackageSpecifier("foo =1.0"))
        self.assertTrue("foo >=1.0, <=1.0" in conda_envfile.PackageSpecifier("foo =1.0"))
        self.assertTrue("foo =1.0" not in conda_envfile.PackageSpecifier("foo =2.0"))
        self.assertTrue("foo >0.9" not in conda_envfile.PackageSpecifier("foo =1.0"))
        self.assertTrue("foo <1.1" not in conda_envfile.PackageSpecifier("foo =1.0"))
        self.assertTrue("foo >0.9, <1.1" not in conda_envfile.PackageSpecifier("foo =1.0"))

        self.assertTrue("foo =1.0" in conda_envfile.PackageSpecifier("foo >=1.0"))
        self.assertTrue("foo =1.0" in conda_envfile.PackageSpecifier("foo >0.9"))

        self.assertTrue("foo =1.0" in conda_envfile.PackageSpecifier("foo >=1.0, <2.0"))
        self.assertTrue("foo =1.0" in conda_envfile.PackageSpecifier("foo >=1.0, <=2.0"))
        self.assertTrue("foo =1.0" in conda_envfile.PackageSpecifier("foo >0.9, <2.0"))
        self.assertTrue("foo =1.0" in conda_envfile.PackageSpecifier("foo >0.9, <=2.0"))

        self.assertTrue("foo >=1.0, <2.0" not in conda_envfile.PackageSpecifier("foo =1.0"))
        self.assertTrue("foo >=1.0, <=2.0" not in conda_envfile.PackageSpecifier("foo =1.0"))
        self.assertTrue("foo >0.9, <2.0" not in conda_envfile.PackageSpecifier("foo =1.0"))
        self.assertTrue("foo >0.9, <=2.0" not in conda_envfile.PackageSpecifier("foo =1.0"))

        self.assertTrue("foo =1.0" not in conda_envfile.PackageSpecifier("foo >1.0"))
        self.assertTrue("foo =1.0" not in conda_envfile.PackageSpecifier("foo <1.0"))

        self.assertTrue("foo <1.0" in conda_envfile.PackageSpecifier("foo <1.0"))
        self.assertTrue("foo <0.9" in conda_envfile.PackageSpecifier("foo <1.0"))
        self.assertTrue("foo <0.9" in conda_envfile.PackageSpecifier("foo <=1.0"))
        self.assertTrue("foo <=0.9" in conda_envfile.PackageSpecifier("foo <1.0"))
        self.assertTrue("foo <=0.9" in conda_envfile.PackageSpecifier("foo <=1.0"))
        self.assertTrue("foo <=1.0" not in conda_envfile.PackageSpecifier("foo <1.0"))
        self.assertTrue("foo <1.0" not in conda_envfile.PackageSpecifier("foo <=0.9"))
        self.assertTrue("foo >=1.0" not in conda_envfile.PackageSpecifier("foo >1.0"))
        self.assertTrue("foo >1.0" not in conda_envfile.PackageSpecifier("foo >=1.1"))
        self.assertTrue("foo" not in conda_envfile.PackageSpecifier("foo <1.0"))

        self.assertTrue("foo >=0.5, <=1.0" in conda_envfile.PackageSpecifier("foo <2.0"))
        self.assertTrue("foo >0.5, <=1.0" in conda_envfile.PackageSpecifier("foo <2.0"))
        self.assertTrue("foo >0.5, <1.0" in conda_envfile.PackageSpecifier("foo <1.0"))
        self.assertTrue("foo <1.0" not in conda_envfile.PackageSpecifier("foo >0.5, <1.0"))
        self.assertTrue("foo <1.0" not in conda_envfile.PackageSpecifier("foo >0.5, <=1.0"))
        self.assertTrue("foo <1.0" not in conda_envfile.PackageSpecifier("foo >=0.5, <1.0"))
        self.assertTrue("foo <1.0" not in conda_envfile.PackageSpecifier("foo >=0.5, <=1.0"))

        self.assertTrue("foo *" in conda_envfile.PackageSpecifier("foo"))
        self.assertTrue("foo =1.*" in conda_envfile.PackageSpecifier("foo >=1.0, <2.0"))
        self.assertTrue("foo =1.*" not in conda_envfile.PackageSpecifier("foo >1.0, <2.0"))

    def test_interpret(self):

        interpret = [
            "foo",
            "foo >1.0",
            "foo >=1.0",
            "foo >=1.0, <2.0",
            "foo >=1.0, <=2.0",
            "foo >1.0, <2.0",
            "foo >1.0, <=2.0",
            "foo =1.0.*",
        ]

        for name in interpret:
            self.assertEqual(str(conda_envfile.PackageSpecifier(name)), name)
            self.assertEqual(str(conda_envfile.PackageSpecifier(name.replace(" ", ""))), name)

        self.assertEqual(
            str(conda_envfile.PackageSpecifier("foo >=1.2.0, <=1.2.0")),
            "foo =1.2.0",
        )

        illegal = [
            "foo >1.2.0, <1.2.0",
            "foo >=1.2.0, <1.2.0",
            "foo >1.2.0, <=1.2.0",
            "foo >=1.3.0, <=1.2.0",
        ]

        for dep in illegal:
            with self.assertRaises(ValueError):
                conda_envfile.PackageSpecifier(dep)


if __name__ == "__main__":

    unittest.main(verbosity=2)



from pathlib import Path

from setuptools import find_packages
from setuptools import setup

project_name = "conda_envfile"

this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(
    name=project_name,
    license="MIT",
    author="Tom de Geus",
    author_email="tom@geus.me",
    description="Inspect/combine conda YAML environment files",
    long_description=long_description,
    long_description_content_type="text/markdown",
    keywords="conda, YAML",
    url=f"https://github.com/tdegeus/{project_name:s}",
    packages=find_packages(exclude=["tests"]),
    use_scm_version={"write_to": f"{project_name}/_version.py"},
    setup_requires=["setuptools_scm"],
    install_requires=["click", "Jinja2", "packaging", "prettytable", "pyyaml"],
    entry_points={
        "console_scripts": [
            f"conda_envfile_merge = {project_name}:_conda_envfile_merge_cli",
            f"conda_envfile_parse = {project_name}:_conda_envfile_parse_cli",
            f"conda_envfile_restrict = {project_name}:_conda_envfile_restrict_cli",
            f"conda_envfile_diff = {project_name}:_conda_envfile_diff_cli",
        ]
    },
)
